from RoboidControl.Messages.IMessage import IMessage
from RoboidControl.Thing import Thing

from typing import Union, Optional
import logging
logger = logging.getLogger(__name__)

class InvestigateMsg():
    """! Message to request details for a Thing
    """
    ## The message ID
    id = 0x81
    ## The length of the message in bytes
    length = 3

    def __init__(self, arg1: Union[bytes, int], thing: Optional[Thing] = None):
        """! Create an investigate message
        @param arg1 Either
        - bytes containing the serialized data; or
        - an int containing the network_id
        @param thing The thing for which the details are requested
        """
        if thing is None:
            ## Create from serialized data
            data = bytes(arg1)
            self.network_id: int = data[1]
            self.thing_id: int = data[2]
        else:
            ## Create from thing details
            self.network_id: int = int(arg1)
            self.thing_id: int = thing.id

    def Serialize(self, ref_buffer: list[bytearray]):
        """!
        @copydoc RoboidControl::Messages::IMessage::IMessage::Serialize
        """

        logger.debug(f'Send DestroyMsg [{self.network_id}/{self.thing_id}]')

        buffer: bytearray = ref_buffer[0]
      
        buffer[0:InvestigateMsg.length] = [
            InvestigateMsg.id,
            self.network_id,
            self.thing_id
        ]
        return InvestigateMsg.length
