from RoboidControl.Messages.IMessage import IMessage
from RoboidControl.Thing import Thing

from typing import Optional, Union
import logging
logger = logging.getLogger(__name__)

class NameMsg(IMessage):
    id = 0x91
    length = 4

    def __init__(self, arg1: Union[bytes, int], thing: Optional[Thing] = None):
        """! Create a NameMsg
        @param arg1 Either:
        - bytes: contianing the serialized data; or
        - int: containgint the network_id 
        @param thing The thing for which the data is communicated
        """
        if thing is None:
            data = bytes(arg1)
            ## The network ID of the thing
            self.network_id: int = data[1]
            ## The ID of the thing
            self.thing_id: int = data[2]
            ## The name
            self.name: str = data[NameMsg.length:]
            ## The length of the encoded name
            self.name_length: int = len(self.name)
            self.name = self.name.decode("utf-8")
        else:
            ## The network ID of the thing
            self.network_id: int = int(arg1)
            ## The ID of the thing
            self.thing_id: int = thing.id
            ## The name
            if thing.name is None:
                self.name: Optional[str] = None
            else:
                self.name: Optional[str] = thing.name.encode('utf-8')
                self.name_length = len(self.name)

    def Serialize(self, buffer_ref: list[bytearray]):
        """!
        @copydoc RoboidControl::Messages::IMessage::IMessage::Serialize
        """ 
        if self.name is None:
            return 0

        buffer: bytearray = buffer_ref[0]

        full_length = NameMsg.length + self.name_length
        if self.name_length == 0 or full_length > len(buffer):
            return 0
        
        logger.debug(f'Send NameMsg [{self.network_id}/{self.thing_id}] {self.name_length}')

        buffer[0:NameMsg.length] = [
            NameMsg.id,
            self.network_id,
            self.thing_id,
            self.name_length
        ]
        # Append the name string
        buffer[NameMsg.length:full_length] = self.name
        return full_length
